/*
 * The code for the Main Window 
 * Copyright (c) 1998 - Stig Erik Sand
 *
 * $Id: PrefDialog.cpp,v 1.20 1998/06/18 22:56:04 stig Exp $
 */

#include "PrefDialog.h"
#include "paintarea.H"
//#include "tools.H"
#include <stdlib.h>
#include <iostream.h>
#include <qfiledlg.h>
#include <qapp.h>
#include <qpopmenu.h>
#include <qkeycode.h>
#include <qradiobt.h>
#include <qmsgbox.h>

#define Inherited PrefDialogData
#define MYAPP_NAME "Macbeth"

PrefDialog::PrefDialog(QWidget* parent, const char* name) :
    Inherited( parent, name ) {

    setCaption( MYAPP_NAME );

    acceptLevelSelect->setCurrentItem(2);
    doActualSave();

    paintarea = new PaintArea(MYAPP_NAME": Picture", this);
    paintarea->hide();

    QPopupMenu *commands = new QPopupMenu();
    CHECK_PTR(commands);
    
    commands->insertItem("&Process picture", this, SLOT(saveVars()));
    commands->insertItem("&Split infile", paintarea, SLOT(splitInfile())); 
    commands->insertSeparator();
    commands->insertItem("&Quit", qApp, SLOT(quit()), CTRL+Key_Q);
    theMenu->insertItem("&Commands", commands);

    QPopupMenu *help = new QPopupMenu;
    CHECK_PTR( help );
    help->insertItem( "&About Macbeth", this, SLOT(about()) );

    theMenu->insertSeparator();
    theMenu->insertItem("&Help", help);
    theMenu->setSeparator( QMenuBar::InWindowsStyle );
     
    connect(quitButton,  SIGNAL(clicked()), qApp,      SLOT(quit()));
    connect(splitButton, SIGNAL(clicked()), paintarea, SLOT(splitInfile()));
}


PrefDialog::~PrefDialog() {

}

void
PrefDialog::saveVars() {

    doActualSave();
    paintarea->startProcess();
    
}

void
PrefDialog::getInfile() {
    
    QString f = QFileDialog::getOpenFileName( ".", "*.jpg");
    if ( !f.isEmpty() ) {
	infileInput->setText(f);
    } 
}

void
PrefDialog::getOutfile() {
    
    QString f = QFileDialog::getSaveFileName( ".", "*.bmp");
    if ( !f.isEmpty() ) {
	outfileInput->setText(f);
    } 

}

void
PrefDialog::getStatfile() {
    
    QString f = QFileDialog::getOpenFileName( ".", "*.mac");
    if ( !f.isEmpty() ) {
	statfileInput->setText(f);
    } 

}

void
PrefDialog::doActualSave() {


    prefs.infilename = infileInput->text();
    if (!prefs.infilename || !strlen(prefs.infilename))
	prefs.infilename = DEFAULT_INFILE;
    COUTPUT("Infile set to: " << prefs.infilename);
    prefs.outfilename = outfileInput->text();
    if (!prefs.outfilename || !strlen(prefs.outfilename))
	prefs.outfilename = DEFAULT_OUTFILE;
    prefs.statfilename = statfileInput->text();
    if (!prefs.statfilename || !strlen(prefs.statfilename))
	prefs.statfilename = DEFAULT_STATFILE;

    const char *newX = newXInput->text();
    const char *newY = newYInput->text();
    const char *newSX = splitXInput->text();
    const char *newSY = splitYInput->text();
    const char *newAlt = altSizeInput->text();
    const char *newCon = maxSizeConstInput->text();
    const char *picX = maxPicXInput->text();
    const char *picY = maxPicYInput->text();
    const char *newAcc = acceptLevelSelect->currentText();
	
    if (!newX || !strlen(newX)) prefs.tileSizeX=DEFAULT_TILEX;
    else prefs.tileSizeX=atoi(newX);
    
    if (!newY || !strlen(newY)) prefs.tileSizeY=DEFAULT_TILEY;
    else prefs.tileSizeY=atoi(newY);

    if (!newAlt || !strlen(newAlt)) prefs.altsize=DEFAULT_ALTSIZE;
    else prefs.altsize=atoi(newAlt);
	
    if (!newSX || !strlen(newSX)) prefs.splitX=PRINTABLE_WIDTH;
    else prefs.splitX=atoi(newSX);
    
    if (!newSY || !strlen(newSY)) prefs.splitY=PRINTABLE_HEIGHT;
    else prefs.splitY=atoi(newSY);
    
    if (!newCon || !strlen(newCon)) prefs.maxsizeconst=DEFAULT_MAXSIZECONST;
    else prefs.maxsizeconst=atof(newCon);

    if (!picX || !strlen(picX)) prefs.maxPicX=DEFAULT_MAXPIC_X;
    else prefs.maxPicX=atoi(picX);
    
    if (!picY || !strlen(picY)) prefs.maxPicY=DEFAULT_MAXPIC_Y;
    else prefs.maxPicY=atoi(picY);

    
    if (!newAcc || !strlen(newAcc)) prefs.acceptlvl=DEFAULT_DBLACCEPT;
    else prefs.acceptlvl=atoi(newAcc);

    
    prefs.disporiginal = dispOrigCheck->isChecked();
    prefs.algorithm    = DARKTEST;
    prefs.contdisp     = contDispCheck->isChecked();
    prefs.printcheck   = printableCheck->isChecked();
    prefs.alw_samesize = sameSizeCheck->isChecked();
    prefs.onlyuseonce  = onlyOnceCheck->isChecked();
    prefs.neverexpand  = neverExpandCheck->isChecked();
}

void
PrefDialog::updateInfile() {

    prefs.infilename = infileInput->text();
    if (!prefs.infilename || !strlen(prefs.infilename))
	prefs.infilename = DEFAULT_INFILE;

}

void
PrefDialog::algoSelect(int algo) {

    if (algo==prefs.algorithm) return;
    
    QRadioButton *bah = (QRadioButton *)algoButtons->find(prefs.algorithm);

    bah->setChecked(FALSE);
    prefs.algorithm=(AlgoType)algo;
    
}

void
PrefDialog::about() {

    QMessageBox::about( this, "About Macbeth v0.3b",
     "This is the Macbeth software package. It is a neat tool if you\n"\
     "plan to make mosaics out of pictures and photos. It was inspired\n"\
     "by a Newsweek article about Silvers who first started making\n"\
     "Photomosaics (tm). He did not want to share his artistic tool, so\n"\
     "I borrowed the general idea (from the Newsweek article, I have\n"\
     "never seen any code or algorithms) to make this artistic tool\n"\
     "available to everyone. If only one painter had a brush we would\n"\
     "never have had so many great paintings, and if only one person\n"\
     "have a mosaicking tool we may lose our chance to actually get a\n"\
     "great many snazzy mosaics. To make the tool \"free\", the software,\n"\
     "its code and documentation has been copylefted under the GNU\n"\
     "General Public License\n\n"\
     "Copyright (C) 1998 Stig E. Sand <stig@kunst.uib.no> \n\n"\
     "Official page: http://kunst.uib.no/~stig/Macbeth/"); 

}


