/*
 * The implementation of the gif-reading
 * Copyright (c) 1998 - Stig Erik Sand
 *
 * $Id: gifread.cpp,v 1.6 1998/08/26 08:11:58 stig Exp $
 */

#include "tools.H"

extern "C" {
#include "gd/gd.h"
/* #include "gd/gdfontmb.h" */ 
}

#include <stdlib.h>
#include <iostream.h>

#include <qimage.h>
#include <qmsgbox.h>

QImage *
PictureTools::makeImage(const char *fname, int thorough) {

    QImage *image;
    FILE *infile = fopen(fname, "rb");
    
    if (!infile && thorough) {
	char buffer[128];
	sprintf(buffer, "%s %s\n\n%s", "Could not find file ",
		fname, "Macbeth will now exit.");
	QMessageBox::critical( 0, "Macbeth", buffer);
	exit(-1);
    }
    else if (!infile) {
	fprintf(stderr, "%s %s\n\n%s", "Could not find file ",
		fname, "Macbeth will now exit.");
	return 0;
    }

    
    const char *whatFormat = strstr(fname, "gif");
    if (whatFormat) {
//	puts("Trying to read");
	gdImagePtr imPtr = gdImageCreateFromGif(infile);
	fclose(infile);
	if (!imPtr) return NULL;
	if (!thorough) {
	    printInfo(fname,imPtr);
	    gdImageDestroy(imPtr);
	    return NULL;
	}
	image = makeImage(imPtr);
	gdImageDestroy(imPtr);
    }
    else if ((whatFormat = strstr(fname, "jpg"))) {
	fclose(infile);
	image = getJpegImage(fname, thorough);
    }
    
    return image;
}

QImage *
PictureTools::makeImage(gdImagePtr imp) {
    
    QImage * newimg = new QImage(imp->sx, imp->sy, 8, 256);
    
    QRgb *foo = newimg->colorTable();
    for (int k=0; k< imp->colorsTotal; k++)
	*foo++ = qRgb(imp->red[k], imp->green[k], imp->blue[k]);

    unsigned char *pah;
    for (int i=0; i<imp->sy; i++) {
	pah=newimg->scanLine(i);
	for (int j=0; j<imp->sx; j++)
	    pah[j] = imp->pixels[j][i];
    }

    QImage *tmp = new QImage(newimg->convertDepth(32));
    
    delete newimg;
    
    return tmp;;
}

void
PictureTools::printInfo(const char *filename, gdImageStruct *imp) {

    cout << "GIF [" << filename << "]\t\t(" << imp->sx << "," << imp->sy << ") - ";

    cout << imp->colorsTotal << " colours" << endl;
    
}
