/*
 * The implementation of the jpeg-reading
 * Copyright (c) 1998 - Stig Erik Sand
 *
 * $Id: jpegread.cpp,v 1.9 1998/08/26 08:11:58 stig Exp $
 */

#include "tools.H"
#include <stdio.h>
#include <stdlib.h>
#include <setjmp.h>
#include <iostream.h>

extern "C" {
#include <jpeglib.h>
#include <jerror.h>    
}

static jmp_buf localjump;
static const char *currentFile;

static void
my_error_exit(j_common_ptr cinfo) {

    /** Handles jpeg-exits.. JERR_NO_SOI should be ignored */
    fprintf(stderr, "Error with file: %s\n\t", currentFile);
    int err_code = cinfo->err->msg_code;
    if (err_code==JERR_NO_SOI) {
	longjmp(localjump,1);
	return;
    }
    else {
	WARNMS(cinfo,err_code);
//	fprintf(stderr, "Fatal jpeg-error (%d)\n", err_code);
//	cinfo->err->(*output_message)(cinfo);
	exit(-1);
    }
}



QImage *
PictureTools::getJpegImage(const char *filename, int thorough) {

    /** Reads in jpeg image.. duh */
    
    FILE *infile = fopen(filename, "rb");
    currentFile=filename;
    jpeg_decompress_struct cinfo;
    jpeg_error_mgr jerr;
	
    cinfo.err = jpeg_std_error(&jerr);
    jpeg_create_decompress(&cinfo);

    jerr.error_exit = my_error_exit;
//    puts("Before");
    jpeg_stdio_src(&cinfo, infile);
//    fprintf(stderr,"Betw. %d and value: %d\n", JERR_NO_SOI, jerr.msg_code);
    
    int val = setjmp(localjump);
    if (!val) {
	jpeg_read_header(&cinfo, TRUE);
    }
    else {
//	fprintf(stderr, "What?\n");
	if (jerr.msg_code==JERR_NO_SOI) {
	    jpeg_destroy_decompress(&cinfo);
	    fclose(infile);
	    return NULL;
	}
    }
    
    if (!thorough) {
	printInfo(filename, cinfo);
	jpeg_destroy_decompress(&cinfo);
	fclose(infile);
	return 0;
    }
//    cinfo.out_color_space = JCS_GRAYSCALE;
//    cinfo.out_color_space = JCS_RGB;
//    cinfo.quantize_colors = TRUE;

    if (cinfo.out_color_space==JCS_GRAYSCALE) {
	fprintf(stderr, "Error: GRAYSCALE %s\n", filename);
	jpeg_destroy_decompress(&cinfo);
	fclose(infile);
	return 0;
    }
    
    jpeg_start_decompress(&cinfo);

    const int blah = cinfo.output_width*cinfo.output_components;
/*    
    cout << "(W: " << cinfo.image_width << ", H: "
	 << cinfo.image_height << ") ";

    cout << "(OCC: " << cinfo.out_color_components << ", OC: "
	 << cinfo.output_components << ", CM: " << cinfo.colormap
	 << ", ACT: " << cinfo.actual_number_of_colors << ") ";
    cout << "(Blah: " << blah << ", Des: " << cinfo.desired_number_of_colors
	 << ")";
    cout << endl;
*/
    
    QImage *pnt = PictureTools::initImage(cinfo.output_width,
					  cinfo.output_height,
					  32, 0);
    JSAMPARRAY buffer = (*cinfo.mem->alloc_sarray)((j_common_ptr) &cinfo,
					JPOOL_IMAGE, blah, 1);   

//    PictureTools::setColours(pnt, cinfo.colormap,
//			     cinfo.actual_number_of_colors);


    for (int i=0; cinfo.output_scanline < cinfo.output_height; i++) {
	jpeg_read_scanlines(&cinfo, buffer, 1);
	PictureTools::copy32Buffer(pnt, buffer[0], blah, i);
    }

    jpeg_finish_decompress(&cinfo);
    jpeg_destroy_decompress(&cinfo);
    fclose(infile);
    
    return pnt;
}


void
PictureTools::printInfo(const char *filename, jpeg_decompress_struct &cinfo) {

    cout << "JPEG[" << filename << "]\t\t("
	 << cinfo.image_width << "," << cinfo.image_height << ") - ";
    if (cinfo.out_color_space==JCS_GRAYSCALE) {
	cout << "GRAYSCALE ";
    }
    
    cout << endl;
       
}
