#ifndef MOSAIC_H
#define MOSAIC_H

/*
 * Class declaration for the actual Mosaic
 * Copyright (c) 1998 - Stig Erik Sand
 *
 * $Id: mosaic.H,v 1.6 1998/08/01 12:43:48 stig Exp $
 */

class TiledPicture;
class AlternativePicture;
class PicturePart;
class QImage;
class QPixmap;
class QPainter;
class QPaintDevice;
class QWidget;
class PrefStruct;
class QPopupMenu;

class Mosaic {
public:
    Mosaic(const QImage *original, const PrefStruct &prefs);
    ~Mosaic();

    int getPictureVal(int tile, int alt);
    QPixmap *getPicture(int tile, int alt);
    QPixmap getColourData(const PicturePart *whichpart);
    int calculateMosaic(QPaintDevice *paint, QPainter *pixpaint,
			QPixmap *targetPixmap, const PrefStruct&);
    int usedPictures(int len) const;
    void displayAlternative(QPainter *paint, int x, int y, const int *pic);
    void pasteInAlternative(QPainter *paint, int tile, int alt);
    void buildPopup(QPopupMenu *pictlist, int whichtile, int altsize, int whichButton); 
    
private:
    /** Length of index of pictures */
    unsigned int indexLen;
    /** Array with number of picture */
    int **picIndexes;
    /** Data about the tiles of the picture */
    TiledPicture *tiledpic;
    /** Irregular tilemap */
    TiledPicture *irregpic;
    /** Alternatives */
    AlternativePicture *alternatives;
    /** The pixmaps read */
    QPixmap *pixmaps;
    /** How many pictures in database */
    int howmany;
    /** Tile sizes */
    unsigned int tileX, tileY;
    
    /** Parts of the original picture */
    const PicturePart *parts;
    /** Parts of the irregular picture */
    const PicturePart *irregparts;

    /** Just a small detail */
    const QPixmap *originalParts;
};

#endif /* MOSAIC_H */
