/*
 * The implementation of the painting area
 * Copyright (c) 1998 - Stig Erik Sand
 *
 * $Id: paintarea.cpp,v 1.27 1998/08/01 12:43:48 stig Exp $
 */


#include "tools.H"
#include "paintarea.H"
#include "mosaic.H"
#include <stdio.h>
#include <stdlib.h>
#include <iostream.h>
#include <qapp.h>
#include <qpixmap.h>
#include <qpainter.h>
#include <qimage.h>
#include <qpopmenu.h>
#include <qcursor.h>
#include <qmsgbox.h>
#include "PrefDialog.h"

PaintArea::PaintArea(const char* name, PrefDialog *pooh)
    : QWidget(0, name,  110592), theMosaic(NULL),
      originalImage(NULL),
      prefs(pooh),
      thepixmap(NULL)
{
    /** Basic constructor */
    
    setCaption( "Ze Mosaic" );

    pictlist = new QPopupMenu;
    
    popmen = new QPopupMenu;
    popmen->insertItem("Pictures", pictlist);
    popmen->insertItem("Save as printable parts", this,
		       SLOT(saveAsPrintable()));
    popmen->insertItem("Quit", qApp, SLOT(quit()));
    connect(pictlist, SIGNAL(activated(int)),
	    this, SLOT(pasteInAlternative(int)));
    
}

PaintArea::~PaintArea() {

    /** Simple destructor */
    
    if (originalImage) delete originalImage;
    if (thepixmap) delete thepixmap;
}

void
PaintArea::displayPicture(QImage *im) {

    /** Just displays the picture on _this_ */
    
    QPixmap pix;
    pix.convertFromImage(*im);
    QPainter paint(&pix);
    bitBlt(this, 0, 0, &pix);
}


void
PaintArea::startProcess() {

    /** Starts the whole show.. */
    
    setCursor(waitCursor);
    const PrefStruct *_prefs = prefs->getPrefs();
    const char *fname = _prefs->infilename;

    COUTPUT("Starting processing on |" << fname << "|");

    originalImage = PictureTools::makeImage(fname);
    picWidth  = originalImage->width();
    picHeight = originalImage->height();
    
    resize(picWidth + MY_BORDER*2, picHeight + MY_BORDER*2);

    thepixmap = new QPixmap(picWidth, picHeight);
    QPainter paint(thepixmap);

    /* Display original */
    if (_prefs->disporiginal) {
	COUTPUT("Displaying original");
	QPixmap rabbit;
	rabbit.convertFromImage(*originalImage);
	paint.drawPixmap(0,0, rabbit);
    }
   
    show();

        
    COUTPUT("Starting calculation");
    if (theMosaic) delete theMosaic;
    theMosaic = new Mosaic(originalImage, *_prefs);
    theMosaic->calculateMosaic(this, &paint, thepixmap, *_prefs);

    bitBlt(this, MY_BORDER, MY_BORDER, thepixmap);
    thepixmap->save(_prefs->outfilename, "BMP");
    
    if (_prefs->printcheck) saveAsPrintable();
    
    setCursor(arrowCursor);
    
}

void
PaintArea::pasteInAlternative(int i) {

    /** Pastes in chosen alternative */
    
    QPainter paint(thepixmap);    
    
    theMosaic->pasteInAlternative(&paint, whichtile, i);
	
    bitBlt(this, MY_BORDER, MY_BORDER, thepixmap);
    thepixmap->save(prefs->getPrefs()->outfilename, DEFAULT_OUTFORMAT);

}

void
PaintArea::paintEvent ( QPaintEvent * ) {
    
    /* dumb implementation */
    
    if (thepixmap)
	bitBlt(this, MY_BORDER, MY_BORDER, thepixmap);

}


void
PaintArea::mousePressEvent ( QMouseEvent *e ) {

    /** Creates a menu when right mouse button is clicked */

    int whichButton = e->button();
    
    if (whichButton == RightButton || whichButton == MidButton) {
	const PrefStruct *_prefs = prefs->getPrefs();
	pictlist->clear();
	QPoint p = mapFromGlobal(QCursor::pos());
	int x = p.x();
	int y = p.y();
	int w = x / _prefs->tileSizeX;
	int h = y / _prefs->tileSizeY;

	int col = picWidth / _prefs->tileSizeX;
	whichtile = col*h+w;

//	COUTPUT("("<<x<<","<<y<<") ["<<w<<","<<h<<"] "<<row<<"%"<<whichtile);
	
	/* dummy put in the right pictures based on tile */
	theMosaic->buildPopup(pictlist, whichtile, _prefs->altsize, whichButton);
	
	popmen->popup(QCursor::pos());
    }
	
}

void
PaintArea::saveAsPrintable() {

    /** Saves the picture in printable pieces */
    
    if (thepixmap) {
	setCursor(waitCursor);

	QImage im = thepixmap->convertToImage();
	QPixmap ya_pix;

	const PrefStruct *_prefs = prefs->getPrefs();
	const int splitX=_prefs->splitX;
	const int splitY=_prefs->splitY;
		
	const int hei = im.height();
	const int wid = im.width();
	int right = wid / splitX;
	const int rrem = wid-right*splitX;
	if (rrem) right++;
	int down = hei / splitY;
	const int rdown = hei-down*splitY;
	if (rdown) down++;
//	COUTPUT("Wid: " << wid << " hei: " << hei); 
	QImage *tmp;
	char fnbuffer[128];

	for (int i=0; i < right; i++) {
	    for (int j=0; j< down; j++) {
		const int sizex = ((i+1)*splitX < wid) ? splitX : wid - i*splitX;
		const int sizey = ((j+1)*splitY < hei) ? splitY : hei - j*splitY;
		const int startx = i*splitX;
		const int starty = j*splitY;
//		COUTPUT("X: " << sizex << " Y: " << sizey);
//		COUTPUT("SX: " << startx << " SY: " << starty);
		tmp = new QImage(sizex, sizey, 32);
		const int num = im.numColors();
		tmp->setNumColors(num);
		QRgb* pah = im.colorTable();
		QRgb* blah = tmp->colorTable();
		for (int m=0; m<num; m++)
		    *blah++ = *pah++;
			
		for (int k=0; k<sizey; k++) {
		    PictureTools::copyBuffer(tmp,
					     im.scanLine(starty+k)+startx,
					     sizex, k);
		}
		ya_pix.convertFromImage(*tmp);
		::sprintf(fnbuffer,"%s.%d.%d.bmp", "partpic", i, j);
		ya_pix.save(fnbuffer, DEFAULT_OUTFORMAT);
		delete tmp;
	    }
	}
	setCursor(arrowCursor);
    }

}


void
PaintArea::splitInfile() {

    /* Splits the given infile */
    
    prefs->updateInfile();
    
    char buffer[128];
    const char *infname = prefs->getPrefs()->infilename;
    if (!strstr(infname, "bmp")) {
	sprintf(buffer, "%s %s\n", infname, "is not a bmp picture");
    }
    else {
	thepixmap = new QPixmap(infname, "BMP");
	saveAsPrintable();
	sprintf(buffer, "%s %s\n", infname, "has been split");
    }
    QMessageBox::information( 0, "Macbeth", buffer);
}
