/*
 * Implementation of Picture-related classes/functions
 * Copyright (c) 1998 - Stig Erik Sand
 *
 * $Id: picture.cpp,v 1.19 1998/08/01 12:43:48 stig Exp $
 */

#include "picture.H"
#include "tools.H"
#include <math.h>
#include <iostream.h>
#include <qimage.h>
#include <qpixmap.h>

#define COLOUR_LIMIT 50
#define COLOUR_LIMIT_SQ 2500

inline int max(int a, int b) {
    return (a>b) ? a : b;
}

inline int min(int a, int b) {
    return (a<b) ? a : b;
}

int
RGBColour::howEqual(const RGBColour& rb) const {
    
    /** Compares "this" colour to "rb" */
    
    int score=0;
    int diffred   = max(rb.red, red)     - min(rb.red, red);
    int diffgreen = max(rb.green, green) - min(rb.green, green);
    int diffblue  = max(rb.blue, blue)   - min(rb.blue, blue);
    if (diffred<COLOUR_LIMIT)
	score += COLOUR_LIMIT_SQ - (diffred   * diffred);
    if (diffgreen<COLOUR_LIMIT)
	score += COLOUR_LIMIT_SQ - (diffgreen * diffgreen);
    if (diffblue<COLOUR_LIMIT)
	score += COLOUR_LIMIT_SQ - (diffblue  * diffblue);

/*    if (abs(rb.red-red)< COLOUR_LIMIT)
	score += COLOUR_LIMIT - abs(rb.red-red);
    
    if (abs(rb.green-green)< COLOUR_LIMIT)
	score += COLOUR_LIMIT - abs(rb.green-green);

    if (abs(rb.blue-blue)< COLOUR_LIMIT)
	score += COLOUR_LIMIT - abs(rb.blue-blue);
*/
    return score;
}

int
RGBColour::dark() const {

    if (red<100 && blue<100 && green<100) return 1;
    return 0;
}

int
RGBColour::light() const {

    if (red>175 && blue>175 && green>175) return 1;
    return 0;
}

int
RGBColour::darkLevel() const {

    return (red+green+blue)/77;
    
}

PicturePart::PicturePart(Coord tl, Coord br) {

/**
 * Constructor who takes top-left and bottom-right
 * coordinates
 */
    
    int width  = br.x-tl.x;
    int height = br.y-tl.y;

    for (int k=0; k < DARK_NUMBER; k++) dark[k] = 0;
    
    const int gridsize = (int)sqrt(COORD_NUMBER);
    int howfar=0;
    for (int x=0; x < gridsize; x++)
	for (int y=0; y < gridsize; y++) {
	    coords[howfar]=Coord(tl.x + (width*x) / gridsize,
				 tl.y + (height*y) / gridsize);
	    
	    howfar++;
	}

}

void
PicturePart::calculateDescr() {

    /** Calculates data about picture */
    
    const int gridsize = (int)sqrt(COORD_NUMBER);
    const int thirdsize = gridsize/3;
    int howfar=0;

//    cout << "Gr: " << gridsize << " third: " << thirdsize << endl;
//    cout << "Val: " << dark[0] << " " << dark[1] << endl;
    
    for (int x=0; x < gridsize; x++)
	for (int y=0; y < gridsize; y++) {

	    if (y < thirdsize) { // Row 1

		if (x < thirdsize)
		    dark[0]+=coords[howfar].colour.darkLevel();
		else if (x >= gridsize-thirdsize)
		    dark[2]+=coords[howfar].colour.darkLevel();
		else
		    dark[1]+=coords[howfar].colour.darkLevel();
	    }
	    else if (y >= gridsize-thirdsize) { // Row 3

		if (x < thirdsize)
		    dark[6]+=coords[howfar].colour.darkLevel();
		else if (x >= gridsize-thirdsize)
		    dark[8]+=coords[howfar].colour.darkLevel();
		else
		    dark[7]+=coords[howfar].colour.darkLevel();
	    }
	    else { // Row 2

		if (x < thirdsize)
		    dark[3]+=coords[howfar].colour.darkLevel();
		else if (x >= gridsize-thirdsize)
		    dark[5]+=coords[howfar].colour.darkLevel();
		else
		    dark[4]+=coords[howfar].colour.darkLevel();
	    }
	    howfar++;
	}
	    
    const int locsize = COORD_NUMBER/9;
//    cout << "duh: " << dark[0] << "-" << dark[1];
    for (int i=0; i < DARK_NUMBER; i++)
	dark[i] = dark[i]/locsize;
//    cout << "#" << locsize << "#" << dark[0] << "-" << dark[1] << endl;
}   


int
PicturePart::equalitySum(const PicturePart& other, int allowed,
			 int checkdlimit) const {

    /** Calculates how equal "this" is to "other" */
    
    if (checkdlimit) {

	ASSERT(dark[0]<30);
	
	for (int k=0; k<DARK_NUMBER; k++) {
//	    cout << "Compare |" << dark[k] << "-"
//		 << other.dark[k] << "|" << endl;
	    if ((dark[k] < (other.dark[k] - allowed)) ||
		(dark[k] > (other.dark[k] + allowed))) return 0;
	}
    }
  
    int tl=0, tr=0, bl=0, br=0;
    const int gridsize = (int)sqrt(COORD_NUMBER);
    const int halfsize = gridsize/2 - 1;
    int h=0;
    int tempsum;
    for (int x=0; x < gridsize; x++) {
	for (int y=0; y < gridsize; y++) {

	    tempsum = coords[h].colour.howEqual(other.coords[h].colour);
	    if (x <= halfsize && y <= halfsize) tl += tempsum;
	    else if (x > halfsize && y <= halfsize) tr += tempsum;
	    else if (x <= halfsize && y > halfsize) bl += tempsum;
	    else if (x > halfsize && y > halfsize) br += tempsum;
	    else cout << "2ehh.." << x << " " << y << endl;
	    
	    h++;
	}
    }

    int totalsum = tl + tr + bl + br;

    return totalsum;
}


TiledPicture::TiledPicture(const QImage *pt, int tX, int tY, int irr) {

    /**
     * Just a constructor which takes an image, tilesizes
     * and a bool whether it is an irregular tiled picture
     */
    
    int addon = 1 + (irr?1:0);
    width = pt->width();  
    height = pt->height();

    int howx = width / tX;
    int howy = height / tY;
     
    howmany = howx * howy;
//    cout <<howmany<<" "<<howx<<" "<<howy<<" "<<width<<" "<<height<<endl;
    
    parts = new PicturePart[howmany+1];
    partPictures = new QPixmap[howmany+1];
    
    int last=0;
//    cout << "#" << irr << "%" << addon << "#" << endl;
    
    int lastx, lasty;
    
    for (int i=0; i < howy; i++) {
	for (int j=0; j < howx; j++) {
	    lastx = (j+addon) * tX - 1;
	    lasty = (i+addon) * tY - 1;
	    if (lastx > width) lastx = width;
	    if (lasty > height) lasty = height;
	    PicturePart foo(Coord(j * tX, i * tY),
			    Coord(lastx, lasty));
//	    cout << " (" << j*tX << ", " << i *tY << ")-(" << lastx
//		 << ", " << lasty << ")" << endl;
	    for (int k=0; k< COORD_NUMBER; k++)
		PictureTools::setColour(foo.coords[k], pt);
	    foo.calculateDescr();

	    partPictures[last] = getPartOfPicture(pt, j*tX, i*tY, tX, tY);
	    parts[last++] = foo;
	}
    }
}

QPixmap
TiledPicture::getPartOfPicture(const QImage *pct, int x, int y, int tX, int tY) {

    QImage *tmp = new QImage(tX, tY, 32);

    unsigned int *pah, *blah;
    for (int i=0; i < tY; ++i) {
	pah = (unsigned int *)pct->scanLine(i+y);
	pah += x; // gamble
	blah = (unsigned int *)tmp->scanLine(i);
	
	memcpy(blah, pah, sizeof(unsigned int)*tX);
    }
    QPixmap out;
    out.convertFromImage(*tmp);
    
    delete tmp;
    
    return out;
}

TiledPicture::~TiledPicture() {
    
    /** Ya Destructor */
    
    delete[] parts;
}


