#ifndef PIC_TOOLS_H
#define PIC_TOOLS_H

/*
 * Headerfile for the toolbox for Pictures
 * Copyright (c) 1998 - Stig Erik Sand
 *
 * $Id: tools.H,v 1.10 1998/08/26 08:11:58 stig Exp $
 */

/* some defines */


#define MY_BORDER 5
#define COUTPUT(arg) cout << arg << endl

#define PRINTABLE_WIDTH 1000
#define PRINTABLE_HEIGHT 1200

#define DEFAULT_STATFILE "stats.mac"
#define DEFAULT_INFILE "infile.jpg"
#define DEFAULT_OUTFILE "outfile.bmp"
#define DEFAULT_OUTFORMAT "BMP"
#define DEFAULT_ALTSIZE 5
#define DEFAULT_TILEX 40
#define DEFAULT_TILEY 40
#define DEFAULT_DBLACCEPT 6
#define DEFAULT_MAXSIZECONST 1.5
#define DEFAULT_MAXPIC_X 2000
#define DEFAULT_MAXPIC_Y 2000
#define IRREGULAR 1

enum AlgoType { DARKTEST=0, DARKCOLTEST, COLTEST };

/** Preference struct */
struct PrefStruct {
public:

    const char *infilename;
    const char *outfilename;
    const char *statfilename;
     
    int tileSizeX;
    int tileSizeY;
    int splitX;
    int splitY;
    
    int disporiginal;
    int contdisp;
    AlgoType algorithm;
    int alw_samesize;
    int neverexpand;
    int altsize;
    int printcheck;
    int acceptlvl;
    int maxPicX; //unused
    int maxPicY; // unused
    int onlyuseonce; //unused
    float maxsizeconst; //unused
};

class QPainter;
class QImage;
class Coord;
class AlternativePicture;
class PicturePart;
struct gdImageStruct;
struct jpeg_decompress_struct;

#include <stdio.h>

class PictureTools {
public:
    
    static void setColour(Coord& pt, const QImage *im);
    static void setColours(QImage *, unsigned char**, int len);
    static void copyBuffer(QImage *, unsigned char*, int len, int index);
    static void copy32Buffer(QImage *, unsigned char*, int len, int index);
    static QImage *makeFlip(QImage *);
    
    /* Analysing functions */
    static AlternativePicture *analysePicture(const char *filename);
    static int analyseDirectory(const char *dirname, FILE *);
    static void analysePictures(const char **names, int num);
    
    /* Duh functions */
    static AlternativePicture* buildPictureTable(int &howmany, const char*);
    static int *findMatch(const AlternativePicture *, int howmany,
			  const PicturePart *, const PrefStruct &,
			  int irr=0);

    /* conversion functions */
    static QImage *makeImage(const char *filename, int thorough=1);
    static QImage *makeImage(gdImageStruct *imp);
    static QImage *getJpegImage(const char *filename, int thorough=1);

    static void printInfo(const char *filename, gdImageStruct *imp);
    static void printInfo(const char *filename, jpeg_decompress_struct &);
    
    /* pre-check functions */
    static int listPictData(const char **names, int num, int thorough=1);
    static int listDirectory(const char *dirname, int thorough=1);
    static int checkPicture(const char *filename, int thorough=1);
    
private:
    /* Private stuff */
    static int isDirectory(const char *path);
    static void paintDot(QPainter *, int r, int g, int b, int, int);
    static QImage *initImage(int w, int h, int d, int c);
};


#endif /* PIC_TOOLS_H */
