/*
 * The implementation of the picture tools
 * Copyright (c) 1998 - Stig Erik Sand
 *
 * $Id: tools.cpp,v 1.12 1998/08/26 08:11:58 stig Exp $
 */

#include "tools.H"
#include "picture.H"
#include <stdio.h>
#include <dirent.h>
#include <sys/stat.h>
#include <iostream.h>
#include <qimage.h>
#include <qpainter.h>

void
PictureTools::setColour(Coord &pt, const QImage *im) {

    /** Gets colour of specific coordinate. Uses nearby pixels too */
    
    int howfar=0;
    int re[5], gr[5], bl[5];
    QRgb temp(im->pixel(pt.x, pt.y));
    re[howfar] = qRed(temp);
    gr[howfar] = qGreen(temp);
    bl[howfar] = qBlue(temp);
    howfar++;
    
    if (im->valid(pt.x-1, pt.y)) {
	temp = im->pixel(pt.x-1, pt.y);
	re[howfar] = qRed(temp);
	gr[howfar] = qGreen(temp);
	bl[howfar] = qBlue(temp);
	howfar++;
    }
    
    if (im->valid(pt.x, pt.y-1)) {	
	temp = im->pixel(pt.x, pt.y-1);
	re[howfar] = qRed(temp);
	gr[howfar] = qGreen(temp);
	bl[howfar] = qBlue(temp);
	howfar++;
    }
    
    if (im->valid(pt.x+1, pt.y)) {
	temp = im->pixel(pt.x+1, pt.y);
	re[howfar] = qRed(temp);
	gr[howfar] = qGreen(temp);
	bl[howfar] = qBlue(temp);
	howfar++;
    }
    
    if (im->valid(pt.x, pt.y+1)) {	
	temp = im->pixel(pt.x, pt.y+1);
	re[howfar] = qRed(temp);
	gr[howfar] = qGreen(temp);
	bl[howfar] = qBlue(temp);
	howfar++;
    }
    
    int avgred=0, avggreen=0, avgblue=0;
    for (int i=0; i<howfar; i++){
	avgred   += re[i];
    	avggreen += gr[i];
	avgblue  += bl[i];
    }
    
    avgred   /= howfar;
    avggreen /= howfar;
    avgblue  /= howfar;
    
    pt.colour = RGBColour(avgred, avggreen, avgblue);
/*    cout << "x(" << x  << ") y(" << y << ") col(" << temp << ") red("
	 << col.red << ") green(" << col.green
	 << ") blue(" << col.blue << ")" << endl;*/
}
	
void
PictureTools::paintDot(QPainter *pnt, int r, int g, int b, int x, int y) {

    /** Paints a dot in specified colour at (x,y) */
    
    if (g!=-1) {
	pnt->setPen(QColor(r, g, b));
	pnt->drawPoint(x,y);
    }
    else {
	pnt->setPen(QColor(r, r, r));
	pnt->drawPoint(x,y);
    }

}

void
PictureTools::setColours(QImage * im, unsigned char** arr, int len) {

    /** Sets colourtable in given image */
    
    QRgb *pah = im->colorTable();
    for (int i=0; i< len; i++)
	*pah++ = qRgb(arr[0][i], arr[1][i], arr[2][i]);
   
}

void
PictureTools::copyBuffer(QImage *im, unsigned char* arr, int len, int index) {

    /** Copies given array to image's scanline */
    
    unsigned char *pah=im->scanLine(index);
    memcpy(pah, arr, len);

}


void
PictureTools::copy32Buffer(QImage *im, unsigned char* arr, int len, int index) {

    /**
     * Copies given array to image's scanline 32 bits.
     * Relatively slow due to qRgb and byte-align
     */
   
    unsigned int *pah = (unsigned int *)im->scanLine(index);
    int j=0;
    for (int i=0; i<len; i+=3) {
	pah[j]=qRgb(arr[i], arr[i+1], arr[i+2]);
	j++;
    }

}


QImage *
PictureTools::initImage(int w, int h, int d, int c) {

    /** Initiates an image with width, height, depth and colour */
    
    QImage * ni = new QImage(w,h,d,c);
    return ni;
}


QImage *
PictureTools::makeFlip(QImage *im) {

    /** Returns a flipped version of given image */
    
    int hei = im->height();
    int wid = im->width();
    
    unsigned int *blah = new unsigned int[wid];
//    COUTPUT("Allocated " << wid+1 << " bytes");
    for (int i=0; i < hei; i++) {
	 unsigned int *pah = (unsigned int *)im->scanLine(i);
	 memcpy(blah, pah, wid*sizeof(unsigned int));
	 
	 for (int j=0; j<wid; ++j) {
	     pah[j]=blah[wid-j-1];
	 }
    }
    
    delete[] blah;
    return im;
}

AlternativePicture *
PictureTools::analysePicture(const char *filename) {

    /**
     * Don't really know.. thinks it take a picture and gets
     * data about it, flips it and returns the AlternativePicture(s)
     */
    
    COUTPUT("Making image \t\t(" << filename << ")");
    QImage *imPtr = makeImage(filename);

    if (!imPtr) {
	COUTPUT("#Error loading image \t(" << filename << ")");
	return NULL;
    }
    
    const int howmany = 2;
//    COUTPUT("Using new..");
    AlternativePicture *tmp = new AlternativePicture[howmany];
    
    tmp[0].flipped=0;
    tmp[1].flipped=1;
    
    for (int i=0; i<howmany; i++) {
	tmp[i].width = imPtr->width();
	tmp[i].height = imPtr->height();
	::strcpy(tmp[i].name, filename);
    }

    int k;
    // normal
    tmp[0].data=PicturePart(Coord(0,0), Coord(tmp[0].width-1,
					      tmp[0].height-1));
    
    for (k=0; k < COORD_NUMBER; k++){
	setColour(tmp[0].data.coords[k],  imPtr);
    }

//    COUTPUT("Starts flip");
    imPtr = makeFlip(imPtr);

    // flipped
    tmp[1].data=PicturePart(Coord(0,0), Coord(tmp[1].width-1,
					      tmp[1].height-1));
    
    for (k=0; k < COORD_NUMBER; k++){
	setColour(tmp[1].data.coords[k],  imPtr);
    }
	
//    COUTPUT("Calcing descr");
    for (int j=0; j<howmany; j++) {
	tmp[j].data.calculateDescr();
    }
    
    delete imPtr;
    
    return tmp;
}

int
PictureTools::isDirectory(const char *path) {
    /**
     * Checks if the <i>path</i> is indeed a directory
     */
  
    struct stat stbuf;

    if (::stat(path, &stbuf)) return 0;
    return ((stbuf.st_mode & S_IFMT) == S_IFDIR) ? 1 : 0;
}

void
PictureTools::analysePictures(const char **filenames, int numfiles) {

    /** Analyses given array of picture and save data to statFile */
    
    AlternativePicture *pics;
    int totalpics=0;
    int howmany;
 
    FILE *statFile = fopen(DEFAULT_STATFILE, "wb");
    fwrite(&totalpics, sizeof(int), 1, statFile);

    /* Want the errmsgs saved */
    FILE *errFile = freopen("error.log", "wb", stderr);
    
    COUTPUT("Size of AltPicture is: " << sizeof(AlternativePicture) << "\n");
    
    for (int i=0; i < numfiles; i++) {
//	COUTPUT("Doing divide and split on(" << i << ") : " << filenames[i]);
	if (isDirectory(filenames[i])) {
	    totalpics += analyseDirectory(filenames[i], statFile);
	}
	else {
	    pics = analysePicture(filenames[i]);
	    if (pics) {
//	    COUTPUT("Done divide");
		howmany=2; // blah
		fwrite(pics, sizeof(AlternativePicture), howmany, statFile);
		totalpics+=howmany;
		delete[] pics;
	    }
	}
    }
    fseek(statFile, 0L, SEEK_SET);
    cout << "\nWritten " << totalpics << " pictures to "
	 << DEFAULT_STATFILE << endl;
    fwrite(&totalpics, sizeof(int), 1, statFile);
    fclose(statFile);
    fclose(errFile);
}

int
PictureTools::analyseDirectory(const char *dirname, FILE *statfile) {
    
    /** Analyses a directory recursively */
    
    int total=0;
    AlternativePicture *pics;
    dirent *direntp;
    DIR *dirp = ::opendir(dirname);
    char buffer[1024];
    for(;(direntp = ::readdir( dirp ));){
	if (*direntp->d_name!='.') {
	    sprintf(buffer, "%s/%s", dirname, direntp->d_name);
//	    cout << "dir: " << buffer << endl;
	    if (isDirectory(buffer)) {
		total+=analyseDirectory(buffer, statfile);
	    }
	    else if (strstr(direntp->d_name, "jpg") ||
		     strstr(direntp->d_name, "gif") ) {
		pics = analysePicture(buffer);
		if (pics) {
		    fwrite(pics, sizeof(AlternativePicture), 2, statfile);
		    total += 2;
		    delete[] pics;
		}
	    }
	    else {
		COUTPUT("Not valid file/dir \t(" << buffer << ")");
	    }
	}
    }
    ::closedir(dirp);

    return total;
}

AlternativePicture *
PictureTools::buildPictureTable(int &howmany, const char *statfilename) {

    /** Returns array of all pictures in statfile */
    
    FILE *statFile = fopen(statfilename, "rb");

    fread(&howmany, sizeof(int), 1, statFile);
    COUTPUT("There are " << howmany << " alternatives");
    AlternativePicture *pics = new AlternativePicture[howmany+1];

    for (int i=0; i < howmany; i++) {
	fread(&pics[i], sizeof(AlternativePicture), 1, statFile);
    }
    	
    fclose(statFile);
    
    return pics;
}

struct KeepAppUp {
    KeepAppUp() : index(-1), value(-1) { }
    int index, value;
};

static void
localSort(KeepAppUp *arr, int arrsize) {

    /** Wowie.. simple selection sort */
    for (int i=0; i<arrsize; i++) {
	KeepAppUp tmp = arr[i];
	int j=i;
	for (; j>0 && tmp.value > arr[j-1].value; j--)
	    arr[j]=arr[j-1];
	arr[j]=tmp;
    }

}


int *
PictureTools::findMatch(const AlternativePicture *sm, int howmany,
			const PicturePart *pp, const PrefStruct & prefs,
			int irr) {

    /**
     * The killer function which decides which pictures are to
     * be immortalized
     */

    int whatallow=BASIC_ALLOW;

    int *retarr = new int[prefs.altsize];
    for (int j=0; j<prefs.altsize; j++) retarr[j] = -1;

    KeepAppUp *values = new KeepAppUp[prefs.altsize+1];
    do {
	int whichalgo = !prefs.algorithm;
	if (whichalgo==DARKCOLTEST &&
	    whatallow==BASIC_ALLOW &&
	    !irr) whichalgo=1;
	for (int i=0; i < howmany; i++) {
	    values[prefs.altsize].index = i;
	    values[prefs.altsize].value = pp->equalitySum(sm[i].data,
							  whatallow,
							  whichalgo);

	    localSort(values, prefs.altsize+1);
	}

//    cout << "[" << (values[0].value/100000) << ", "
//	 << (values[arrsize-1].value/100000) << "]" << endl;

	if (!irr || (values[0].value/100000) >= prefs.acceptlvl) {
    
	    for (int k=0; k < prefs.altsize; k++) {
		if (values[k].value>0)
		    retarr[k] = values[k].index;
		else {
		    retarr[k] = -1;
		}
	    }
	}
	else {
	    for (int j=0; j<prefs.altsize; j++) retarr[j] = -1;
	}
	whatallow++;
    } while (retarr[0]==-1&&!irr);
    
    cout.flush();
    delete[] values;

    return retarr;

}

int
PictureTools::listPictData(const char **filenames, int num, int thorough) {

    /** Lists info on pictures */

    int totalpics=0;
 
    for (int i=0; i < num; i++) {

	if (isDirectory(filenames[i])) {
	    totalpics += listDirectory(filenames[i], thorough);
	}
	else {
	    checkPicture(filenames[i], thorough);
	}
    }
    COUTPUT("Read " << totalpics << " pictures");
    return totalpics;
}

int
PictureTools::listDirectory(const char *dirname, int thorough) {
    
    /** Lists a directory recursively */

    int total=0;
    dirent *direntp;
    DIR *dirp = ::opendir(dirname);
    char buffer[1024];
    for(;(direntp = ::readdir( dirp ));){
	if (*direntp->d_name!='.') {
	    sprintf(buffer, "%s/%s", dirname, direntp->d_name);
//	    cout << "dir: " << buffer << endl;
	    if (isDirectory(buffer)) {
		total+=listDirectory(buffer, thorough);
	    }
	    else if (strstr(direntp->d_name, "jpg") ||
		     strstr(direntp->d_name, "gif") ) {
		checkPicture(buffer, thorough);
	    }
	}
	else {
	    COUTPUT("Not valid file/dir \t(" << buffer << ")");
	}
    }

    ::closedir(dirp);

    return total;
}

int
PictureTools::checkPicture(const char *filename, int thorough) {

    /**
     * Don't really know.. thinks it take a picture and gets
     * data about it, flips it and returns the AlternativePicture(s)
     */
//    static int i=0;
    if (thorough)
	COUTPUT("Checking image \t\t(" << filename << ")");
    QImage *imPtr = makeImage(filename, thorough);
//    char buf[1024];
    if (!imPtr && thorough) {
	COUTPUT("#Error loading image \t(" << filename << ")");
	return 0;
    }
    else {
	/*
	  i++;
	  sprintf(buf, "%d.bmp", i);
	  int val = imPtr->save(buf, "BMP");
	  if (val!=TRUE) cout << "No save\n";
	*/
    }
    int foo = (int)imPtr;
    delete imPtr;
    
    return foo;
}


